#!/usr/bin/env python
# cardinal_pythonlib/typetests.py

"""
===============================================================================

    Original code copyright (C) 2009-2021 Rudolf Cardinal (rudolf@pobox.com).

    This file is part of cardinal_pythonlib.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        https://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

===============================================================================

**Short functions to test the type/value of an object.**

"""

from typing import Any, Iterable


# =============================================================================
# Testers/validators
# =============================================================================

def is_integer(s: Any) -> bool:
    """
    Is the parameter an integer?
    """
    try:
        int(s)
        return True
    except ValueError:
        return False


def raise_if_attr_blank(obj: Any, attrs: Iterable[str]) -> None:
    """
    Raise an :exc:`Exception` if any of the attributes of ``obj`` named in
    ``attrs`` is ``None`` or is ``''``.
    """
    for a in attrs:
        value = getattr(obj, a)
        if value is None or value == "":
            raise Exception(f"Blank attribute: {a}")


# =============================================================================
# bool
# =============================================================================

def is_false(x: Any) -> bool:
    """
    Positively false? Evaluates: ``not x and x is not None``.
    """
    # beware: "0 is False" evaluates to False -- AVOID "is False"!
    # ... but "0 == False" evaluates to True
    # https://stackoverflow.com/questions/3647692/
    # ... but comparisons to booleans with "==" fail PEP8:
    # http://legacy.python.org/dev/peps/pep-0008/
    # ... so use e.g. "bool(x)" or "x" or "not x"
    # http://google-styleguide.googlecode.com/svn/trunk/pyguide.html?showone=True/False_evaluations#True/False_evaluations  # noqa
    return not x and x is not None



#!/usr/bin/env python
# cardinal_pythonlib/run_all_tests.py

"""
===============================================================================

    Original code copyright (C) 2009-2021 Rudolf Cardinal (rudolf@pobox.com).

    This file is part of cardinal_pythonlib.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        https://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

===============================================================================

**Run all tests across the library.**

"""

import subprocess
import os


THIS_DIR = os.path.dirname(os.path.realpath(__file__))


def run_all_unit_tests() -> None:
    cmdargs = [
        "python", "-m", "unittest", "discover", "-s", THIS_DIR, "-p", "*.py"
    ]
    subprocess.check_output(cmdargs)


if __name__ == "__main__":
    run_all_unit_tests()



#!/usr/bin/env python
# cardinal_pythonlib/version_string.py

"""
===============================================================================

    Original code copyright (C) 2009-2021 Rudolf Cardinal (rudolf@pobox.com).

    This file is part of cardinal_pythonlib.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        https://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

===============================================================================

**Current version number of this library.**

NOTE: this file must be importable by setup.py during package installation and
must therefore have NO DEPENDENCIES (e.g. semantic_version).

For changelog, see changelog.rst

"""

VERSION_STRING = '1.1.13'
# Use semantic versioning: http://semver.org/



#!/usr/bin/env python
# cardinal_pythonlib/tools/list_all_file_extensions.py

"""
===============================================================================

    Original code copyright (C) 2009-2021 Rudolf Cardinal (rudolf@pobox.com).

    This file is part of cardinal_pythonlib.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        https://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

===============================================================================

**Command-line tool to enumerate all file extensions found within a path.**

"""


import argparse
import logging
import os
from typing import List

from cardinal_pythonlib.logs import (
    BraceStyleAdapter,
    main_only_quicksetup_rootlogger,
)

log = BraceStyleAdapter(logging.getLogger(__name__))


def list_file_extensions(path: str, reportevery: int = 1) -> List[str]:
    """
    Returns a sorted list of every file extension found in a directory
    and its subdirectories.

    Args:
        path: path to scan
        reportevery: report directory progress after every *n* steps

    Returns:
        sorted list of every file extension found

    """
    extensions = set()
    count = 0
    for root, dirs, files in os.walk(path):
        count += 1
        if count % reportevery == 0:
            log.debug("Walking directory {}: {!r}", count, root)
        for file in files:
            filename, ext = os.path.splitext(file)
            extensions.add(ext)
    return sorted(list(extensions))


def main() -> None:
    """
    Command-line processor. See ``--help`` for details.
    """
    main_only_quicksetup_rootlogger(level=logging.DEBUG)
    parser = argparse.ArgumentParser()
    parser.add_argument("directory", nargs="?", default=os.getcwd())
    parser.add_argument("--reportevery", default=10000)
    args = parser.parse_args()
    log.info("Extensions in directory {!r}:", args.directory)
    print("\n".join(repr(x) for x in
                    list_file_extensions(args.directory,
                                         reportevery=args.reportevery)))


if __name__ == '__main__':
    main()
